var BOSH_SERVICE = 'http://localhost:5280/http-bind/';

Strophe.addNamespace("ADHOC", "http://jabber.org/protocol/commands");

var localJID = null;
var connection   = null;
var sessionid = null;
var cmdNode = null;

function log(msg) {
    var entry = $('<div></div>').append(document.createTextNode(msg));
    $('#log').append(entry);
}

function rawInput(data) {
    log('RECV: ' + data);
}

function rawOutput(data) {
    log('SENT: ' + data);
}

function onConnect(status) {
    if (status == Strophe.Status.CONNECTING) {
        log('Strophe is connecting.');
    } else if (status == Strophe.Status.CONNFAIL) {
        log('Strophe failed to connect.');
        showConnect();
    } else if (status == Strophe.Status.DISCONNECTING) {
        log('Strophe is disconnecting.');
    } else if (status == Strophe.Status.DISCONNECTED) {
        log('Strophe is disconnected.');
        showConnect();
    } else if (status == Strophe.Status.AUTHFAIL) {
        log('Authentication failed');
        if (connection) {
            connection.disconnect();
        }
    } else if (status == Strophe.Status.CONNECTED) {
        log('Strophe is connected.');
        checkFeatures();
    }
}

function addNote(elem, text, type) {
    if (!type) {
       type = "info";
    }
    $(elem).append("<p class='" + type + "Note'>" + text + "</p>");
}

function addForm(elem, x) {
    var form = $("<form/>");
    form.submit(function(){return false;});
    var fieldset = $("<fieldset/>");
    form.append(fieldset);
    $("<legend/>").text($(x).find("title").text()).appendTo(fieldset);
    $("<p/>").text($(x).find("instructions").text()).appendTo(fieldset);
    $(x).find("field").each(function(){
            var item = null;
            var type = $(this).attr("type");
            if($(this).attr("label")) {
                $("<label/>").text($(this).attr("label")).attr("for", $(this).attr("var")).appendTo(fieldset);
            }
            switch(type) {
            case "hidden":
                item = $("<input type='hidden'/>");
                break;
            case "boolean":
                item = $("<input type='checkbox'/>");
                break;
            case "text-multi":
                item = $("<textarea/>");
                break;
            case "text-single":
                item = $("<input type='text'/>");
                break;
            case "fixed":
                item = $("<input type='text'/>").attr("readonly",true);
                break;
            case "jid-multi":
                item = $("<textarea/>");
                break;
            case "jid-single":
                item = $("<input type='text'/>");
                break;
            case "list-multi":
                item = $("<select multiple='multiple'/>");
                $(this).find("option").each(function(){
                        $("<option/>").val($(this).find("value").text()).text($(this).attr("label")).appendTo(item);
                        });
                break;
            case "list-single":
                item = $("<select/>");
                $(this).find("option").each(function(){
                        $("<option/>").val($(this).find("value").text()).text($(this).attr("label")).appendTo(item);
                        });
                break;
            case "text-private":
                item = $("<input type='password'/>");
                break;
            default:
                item = $("<input/>");
            }
						item.addClass("df-item");
            if ($(this).find("value")) {
                var value = null;
                if ((type == "text-multi") || (type == "jid-multi")) {
                    value = "";
                    $(this).find("value").each(function() {
                        value = value + $(this).text() + "\n";
                    });
                    item.val(value);
                } else if (type == "list-multi") {
                    value = new Array();
                    $(this).find("value").each(function() {
                        value[value.length] = $(this).text();
                    });
                } else {
                    item.val($(this).find("value").text());
                }
            }
            if ($(this).attr("var")) {
                item.attr("name", $(this).attr("var"));
                item.attr("id", $(this).attr("var"));
            }
            fieldset.append(item);
            fieldset.append("<br/>");
    });
    $(elem).append(form);
}

function serializeToDataform(form, st) {
	st.c("x", {"xmlns":"jabber:x:data", "type": "submit"});
	$(form).find(".df-item").each(function(){
		st.c("field", {"var": $(this).attr("name")});
		if(this.nodeName.toLowerCase() == "select" && this.multiple) {
			for(var i = 0; i < this.options.length; i++)
				if(options[i].selected)
					st.c("value").text(options[i]).up();
		} else if(this.nodeName.toLowerCase() == "textarea") {
			var sp_value = this.value.split(/\r\n?|\r/g);
			for(var i = 0; i < sp_value.length; i++)
				st.c("value").text(sp_value[i]).up();
		} else {
			// if this has value then
				st.c("value").text($(this).val()).up();
		}
		st.up();
	});
	st.up();
}

function displayResult(result) {
    var status = $(result).find("command").attr("status");

    $("#output *").remove();
    $(result).find("command > *").each(function(index, e) {
        if ($(e).is("note")) {
            addNote("#output", $(e).text(), $(e).attr("type"));
        } else if ($(e).is("x[xmlns=jabber:x:data]")) {
            addForm("#output", e);
        }
    });
    if (status == "executing") {
        $("#output").append("<input type='button' disabled='true' id='prevButton' value='Prev'/>"+
                            "<input type='button' disabled='true' id='nextButton' value='Next'/>"+
                            "<input type='button' disabled='true' id='completeButton' value='Complete'/>"+
                            "<input type='button' id='executeButton' value='Execute'/>"+
                            "<input type='button' id='cancelButton' value='Cancel'/>");
        for (kind in ['prev', 'next', 'complete']) {
            if ($(result).find('actions ' + kind).length > 0)
                $('#' + kind + 'Button').attr("disabled", "false");
        }
        $('#cancelButton').bind("click", function() {
            var cancelIQ = $iq({ type: "set", to: "localhost", id: connection.getUniqueId() })
                .c("command", { xmlns: Strophe.NS.ADHOC, node: cmdNode, sessionid: sessionid, action: "cancel" });
            cmdNode = null
            sessionid = null;
            connection.sendIQ(cancelIQ, displayResult);
        });
    } else {
        input = $("<input type='button' value='Restart'/>").bind("click", function() {
            $('#output *').remove();
            sessionid = null;
            cmdNode = null;
            getCommandNodes();
        });
        $("#output").append(input);
    }
}

function runCommand() {
    cmdNode = $(this).attr("id"); // Save not of executed command (in global)
    var execIQ = $iq({ type: "set", to: "localhost", id: connection.getUniqueId() })
        .c("command", { xmlns: Strophe.NS.ADHOC, node: cmdNode, action: "execute" });
    connection.sendIQ(execIQ, function(result) {
        sessionid = $(result).find("command").attr("sessionid");
        displayResult(result);
    });
}

function getCommandNodes() {
    var nodesIQ = $iq({ type: "get", to: "localhost", id: "nodes1" }).c("query", {xmlns: Strophe.NS.DISCO_ITEMS, node: Strophe.NS.ADHOC});
    connection.sendIQ(nodesIQ, function(result) {
        $('#output').append("<ul id='items'></ul>");
        $(result).find("item").each(function(index, e) {
            item = $("<li id='" + $(e).attr("node") + "'>" + $(e).attr("name") + "</li>").bind("click", runCommand);
            $("#items").append(item);
        });
    });
}

function checkFeatures() {
    featureIQ = $iq({ type: "get", to: "localhost", id: "features1" }).c("query", {xmlns: Strophe.NS.DISCO_INFO});
    connection.sendIQ(featureIQ, function(result) {
        if ($(result).find("feature[var='" + Strophe.NS.ADHOC + "']").length > 0) {
            $('#output').append("<p>This entitiy does support AdHoc commands</p>");
        } else {
            $('#output').append("<p>This entitiy does NOT support AdHoc commands</p>");
        }
    });
    getCommandNodes();
}

function showConnect() {
    var jid = $('#jid');
    var pass = $('#pass');
    var button = $('#connect').get(0);        

    button.value = 'connect';
    pass.show();
    jid.show();
    $('label').show();
    $('#output *').remove();
    return false;
}

function showDisconnect() {
    var jid = $('#jid');
    var pass = $('#pass');
    var button = $('#connect').get(0);        

    button.value = 'disconnect';
    pass.hide();
    jid.hide();
    $('label').hide();
    return false;
}

$(document).ready(function () {
    connection = new Strophe.Connection(BOSH_SERVICE);
    connection.rawInput = rawInput;
    connection.rawOutput = rawOutput;

    $("#log_toggle").click(function () {
        $("#log").toggle();
      });

    $('#cred').bind('submit', function () {
        var button = $('#connect').get(0);
        var jid = $('#jid');
        var pass = $('#pass');        
        localJID = jid.get(0).value;

        if (button.value == 'connect') {
            showDisconnect();
            $('#log *').remove();
            connection.connect(localJID,
               pass.get(0).value,
               onConnect);
        } else {
            connection.disconnect();
        }
        return false;
    });
});

onunload = function() {
    if (connection) {
        connection.disconnect();
    }
}

